/*
        Vegetate.inc
        Vegetation Generator Include File for the POV-Ray SuperPatch v3.1e

        Created by Mark Wagner, 21 June 1999
        Updated by Mark Wagner, 10 July 1999
                Added Gaussian distribution for types and position
        Updated by Mark Wagner, 17 July 1999
                Changed defaults for VegeMinBox and VegeMaxBox
        Updated by Mark Wagner, 31 August 1999
                Fixed several bugs in ExclusionBox code
                Made 'Verbose' mode far more verbose
        Updated by Mark Wagner, 14 September 1999
                Fixed bugs in ExclusionCircle code
                Added 'Align' option 

        This include file places a number of objects on the surface of another
object.  These objects are selected at random from a user-defined array of 
objects, and are placed within a user-defined region, rotated around the sky
vector and scaled randomly, both within user-specified boundaries.  Options 
also exist for specifying a 'water level' altitude below which objects are not
placed, and a 'timberline' above which objects are not placed.  Maximum slopes
for the placement of objects can also be specified.  Collision detection 
between objects is supported.

        This include file can be used for such things as placing a forest on a
heightfield, placing bugs on a car windshield, penguins on a iceberg or even
craters on a planet!

Note: In order for the code to work, the sky vector must be 
aligned with the x, y or z axes.
 
Note: The random scaling feature currently favors the generation of objects 
that are larger than average.


*/

//Macro for gaussian distribution of random numbers
#macro RandGauss()
        ((rand(VegeSeed)+rand(VegeSeed)+rand(VegeSeed)+rand(VegeSeed)+rand(VegeSeed)+rand(VegeSeed))/6)
#end

//Macro to align an object with the surface normal

#macro Reorient(AXIS1,AXIS2)
        #local VX1=vnormalize(AXIS1);
        #local VX2=vnormalize(AXIS2);
        #local VY=vnormalize(vcross(VX1,VX2));
        #if (vlength(VY)>0)
                #local VZ1=vnormalize(vcross(VX1,VY));
                #local VZ2=vnormalize(vcross(VX2,VY));
                matrix < VX1.x, VY.x,VZ1.x, VX1.y,VY.y,VZ1.y, VX1.z,VY.z, VZ1.z, 0,0,0 >
                matrix < VX2.x,VX2.y,VX2.z,  VY.x,VY.y, VY.z, VZ2.x,VZ2.y,VZ2.z, 0,0,0 >
        #end
#end

//Default object
#local CHEAPPINE = union{
        cylinder{ 0,y*.25,.03 pigment{ color<0.647059,0.164706,0.164706> } }
        cone{ y*.25,.25,y*1,0 pigment{ color<0,1,0> } }
} //A very simple pine tree



//Check defines and set needed defaults

#ifndef(VegeObject) //Object to place stuff on
        #error "No object specified to vegetate.  Cannot continue\n"
#end
#ifndef(VegeHeight) //Height to fire 'trace' rays from
        #ifdef(Verbose)
                #warning "No value specified for VegeHeight.  Default value\nof 1,000,000 may not work properly\n"
        #end
        #declare VegeHeight = 1000000;
#end
#ifndef(VegeNumber) //Number of objects to place
        #declare VegeNumber = 100;
#end
#ifndef(VegeSeed) //Random seed
        #declare VegeSeed = seed(1);
#end
#ifndef(VegePosGauss)
        #declare VegePosGauss = false;
#end
#ifndef(VegeMinScale)
        #declare VegeMinScale = 1;
#end
#ifndef(VegeMaxScale)
        #declare VegeMaxScale = 1;
#end
#ifndef(VegeRotate)
        #declare VegeRotate = true;
#end
#ifndef(SkyVector) //Which way is up?
        #declare SkyVector = y;
#end
#declare SkyVector = vnormalize(SkyVector); //Normalize the sky vector
#ifndef(VegeList) //The list of available objects
        #declare VegeList = array[1]
        #declare VegeList[0] = object{ CHEAPPINE }
#end
#ifndef(VegeTypesGauss)
        #declare VegeTypesGauss = false;
#end
#ifndef(VegeAlign)
        #declare VegeAlign = false;
#end

#declare VEGETypes = dimension_size( VegeList, 1); //How many objects

#ifndef(VegeCollide) //Do collision detection?
        #declare VegeCollide = false;
#end
#ifndef(VegeRadius) //Radius for collision detection.  Assumes that objects are all the same size and
        #declare VegeRadius = 1; //are approximately round.
#end
#if(VegeCollide = true)
        #if(VegePosGauss = true)
                #warning "Using collision detection with Gaussian distribution of objects\nmay result in extremely long parsing times\n"
        #end
#end

#ifdef(VegeExcludeCircle)
        #if(dimensions(VegeExcludeCircle) != 2)
                #error "VegeExcludeCircle array must be a two-dimensional array of 2D vectors\n"
        #end
        #local VEGENumExcludeCircle = dimension_size(VegeExcludeCircle, 1);
#end
#ifdef(VegeExcludeBox)
        #if(dimensions(VegeExcludeBox) != 2)
                #error "VegeExcludeBox array must be a two-dimensional array of 2D vectors\n"
        #end
        #local VEGENumExcludeBox = dimension_size(VegeExcludeBox, 1);
#end
#ifdef(VegeExcludeHeight)
        #if(dimensions(VegeExcludeHeight) != 1)
                #error "VegeExcludeHeight array must be a two-dimensional array of floats\n"
        #end
        #local VEGENumExcludeHeight = dimension_size(VegeExcludeHeight, 1);
#end

//Determine the directions of the 'x' and 'y' vectors
#if(SkyVector.y != 0 ) //parallel or antiparallel to y
        #local X_DIR = x;
        #local Y_DIR = z;
#end
#if(SkyVector.z != 0) //same for z
        #local X_DIR = x;
        #local Y_DIR = y;
#end
#if(SkyVector.x != 0) //same for x
        #local X_DIR = y;
        #local Y_DIR = z;
#end
        
#ifndef(VegeMinBox) //One corner of the bounding box for placing the objects.  Coords are interpreted as
        #declare VegeMinBox = <vdot( min_extent(VegeObject), X_DIR),vdot( min_extent(VegeObject), Y_DIR)>; //being perpendicular to the sky vector
#end
#ifndef(VegeMaxBox) //The other corner of the box
        #declare VegeMaxBox = <vdot( max_extent(VegeObject), X_DIR),vdot( max_extent(VegeObject), Y_DIR)>; //being perpendicular to the sky vector
#end

//Convert bounding box into x+y minimums and ranges
#local VEGE_MINX = min( VegeMinBox.u, VegeMaxBox.u );
#local VEGE_XRANGE = abs(VegeMinBox.u - VegeMaxBox.u );
#local VEGE_MINY = min( VegeMinBox.v, VegeMaxBox.v );
#local VEGE_YRANGE = abs(VegeMinBox.v - VegeMaxBox.v );

#ifdef(Verbose)
        #warning concat( "\n", str( VEGE_MINX, 0, 2), str(VEGE_XRANGE, 0, 2), str(VEGE_MINY,0,2), str(VEGE_YRANGE,0,2), "\n")
#end

//Convert max and min scales to min and range
#local VEGE_SCALE_MIN = min(VegeMinScale, VegeMaxScale); //use min as a safety precaution
#local VEGE_SCALE_RANGE = abs(VegeMinScale - VegeMaxScale);

#local VEGENUM = array[VegeNumber]   //The number of the object type to use for this object
#local VEGETrans = array[VegeNumber] //A transform to move it to its final location and orientation
#if(VegeCollide)
        #local VEGELoc = array[VegeNumber]   //The location of the object for collision detection
#end

//Zero statistic counters
#local VEGEReject = 0;
#local VEGEHeightR = 0;
#local VEGEPosR = 0;
#local VEGEOtherR = 0;


//Error checking
#ifdef(VegeWaterLevel)
        #ifdef(VegeTimberLine)
                #if(VegeWaterLevel > VegeTimberLine)
                        #error "Water level is above timber line.  Unable to place any objects\n"
                #end
        #end
#end

#local VEGE_Vect = <0,0,0>;

#local VEGEcounter = 0;
#while( VEGEcounter < VegeNumber )
        #local VEGE_OK = false;
        #while(!VEGE_OK) //Do while
                #local VEGE_OK = true; //To fake a #do...#while loop
                #if(VegePosGauss)
                        #local VEGE_X = RandGauss()*VEGE_XRANGE + VEGE_MINX;
                        #local VEGE_Y = RandGauss()*VEGE_YRANGE + VEGE_MINY;
                #else
                        #local VEGE_X = rand(VegeSeed)*VEGE_XRANGE + VEGE_MINX;
                        #local VEGE_Y = rand(VegeSeed)*VEGE_YRANGE + VEGE_MINY;
                #end
                #ifdef(Verbose)
                        #warning concat( str( VEGE_X, 0, 2 ), str( VEGE_Y, 0, 2 ), "\n" )
                #end
                #local VEGE_Z = vdot(trace( VegeObject,X_DIR*VEGE_X+Y_DIR*VEGE_Y+SkyVector*VegeHeight ,-SkyVector, VEGE_Vect), SkyVector); //Height of object at point of intersection  
                #if(VEGE_Z = 0 & VEGE_Vect.x = 0 & VEGE_Vect.y = 0 & VEGE_Vect.z = 0)
                        #warning "rejected -- missed object\n"
                        #local VEGE_OK = false;
                        #local VEGEOtherR = VEGEOtherR + 1;
                #end
                #ifdef(VegeWaterLevel)
                        #if( VEGE_Z < VegeWaterLevel & VEGE_OK)
                                #ifdef(Verbose)
                                        #warning "Rejected -- too low\n"
                                #end
                                #local VEGE_OK = false;
                                #local VEGEHeightR = VEGEHeightR + 1;
                        #end
                #end
                #ifdef(VegeTimberLine)
                        #if(VEGE_Z > VegeTimberLine & VEGE_OK)
                                #ifdef(Verbose)
                                        #warning "Rejected -- too high\n"
                                #end
                                #local VEGE_OK = false;
                                #local VEGEHeightR = VEGEHeightR + 1;
                        #end
                #end
                #ifdef(VegeMaxSlope)
                        //#if( acos(vdot(VEGE_Vect, SkyVector)/(vlength(VEGE_Vect)*vlength(SkyVector)))*(180/pi) > VegeMaxSlope)
                        #local SlopeTest = acos(vdot(VEGE_Vect, SkyVector)/(vlength(VEGE_Vect)*vlength(SkyVector)))*(180/pi); //Ge 20.11.01
                        #if (SlopeTest > VegeMaxSlope) //Ge 20.11.01
                        #if (SlopeTest < 180-VegeMaxSlope) //Ge 20.11.01
                                #ifdef(Verbose)
                                        #warning concat (str(SlopeTest,0,2), "Rejected -- too steep\n")
                                #end
                                #local VEGE_OK = false;
                                #local VEGEOtherR = VEGEOtherR + 1;
                        #end //Ge 20.11.01
                        #end
                #end
                #local VEGE_Temp = VEGE_X*x+VEGE_Y*y+VEGE_Z*z;
                #ifdef(VegeExcludeCircle)
                        #if(VEGE_OK)
                                //Test to see if it is in an exclusion zone
                                #ifdef(Verbose)
                                        #warning "Testing exclusion circle..."
                                #end
                                #local VEGEcounter2 = 0;
                                #while( VEGEcounter2 < VEGENumExcludeCircle)
                                        #if( pow((VEGE_X-VegeExcludeCircle[VEGEcounter2][0].u),2)+pow((VEGE_Y-VegeExcludeCircle[VEGEcounter2][0].v),2) < VegeRadius+VegeExcludeCircle[VEGEcounter2][1].u)
                                                #ifdef(Verbose)
                                                        #warning "failed\n"
                                                #end
                                                #local VEGE_OK = false;
                                                #local VEGEcounter2 = VEGENumExcludeCircle; //leave loop
                                                #local VEGEPosR = VEGEPosR + 1; //Increment statistic counter
                                        #end
                                        #local VEGEcounter2 = VEGEcounter2 + 1;                 
                                #end
                                #ifdef(Verbose)
                                        #if(VEGE_OK)
                                                #warning "passed\n"
                                        #end
                                #end
                        #end
                #end                                                    
                #ifdef(VegeExcludeBox)
                        #if(VEGE_OK)
                                //Test to see if it is in an exclusion box
                                #ifdef(Verbose)
                                        #warning "Testing exclusion box..."
                                #end
                                #local VEGEcounter2 = 0;
                                #while( VEGEcounter2 < VEGENumExcludeBox)
                                        #if( (VEGE_X < max(VegeExcludeBox[VEGEcounter2][0].u, VegeExcludeBox[VEGEcounter2][1].u)) & (VEGE_X > min(VegeExcludeBox[VEGEcounter2][1].u, VegeExcludeBox[VEGEcounter2][0].u)) )
                                                #if( (VEGE_Y < max(VegeExcludeBox[VEGEcounter2][0].v, VegeExcludeBox[VEGEcounter2][1].v)) & (VEGE_Y > min(VegeExcludeBox[VEGEcounter2][0].v, VegeExcludeBox[VEGEcounter2][1].v)) )
                                                        #ifdef(Verbose)
                                                                #warning "failed\n"
                                                        #end
                                                        #local VEGE_OK = false;
                                                        #local VEGEcounter2 = VEGENumExcludeBox; //leave loop
                                                        #local VEGEPosR = VEGEPosR + 1; //Increment statistic counter
                                                #end
                                        #end
                                        #local VEGEcounter2 = VEGEcounter2 + 1;
                                #end
                                #ifdef(Verbose)
                                        #if(VEGE_OK)
                                                #warning "OK\n"
                                        #end
                                #end
                        #end
                #end
                #ifdef(VegeExcludeHeight)
                        #if(VEGE_OK)
                                #ifdef(Verbose)
                                        #warning "Testing exclusion height..."
                                #end
                                #local VEGEcounter2 = 0;
                                #while( VEGEcounter2 < VEGENumExcludeHeight )
                                        #if(VEGE_Z = VegeExcludeHeight[VEGEcounter2])
                                                #ifdef(Verbose)
                                                        #warning "failed\n"
                                                #end
                                                #local VEGE_OK = false;
                                                #local VEGEcounter2 = VEGENumExcludeHeight;
                                                #local VEGEHeightR = VEGEHeightR + 1; //Increment statistic counter
                                        #end
                                        #local VEGEcounter2 = VEGEcounter2 + 1;
                                #end
                                #ifdef(Verbose)
                                        #if(VEGE_OK)
                                                #warning "OK\n"
                                        #end
                                #end
                        #end
                #end                                                                                    
                #if( VegeCollide & VEGE_OK)
                        #local VEGEcounter2 = 0;
                        #warning "Collision testing..."
                        //Test current location to see if it has collided with another location
                        #while(VEGEcounter2 < VEGEcounter)
                                #if( pow((VEGE_X-VEGELoc[VEGEcounter2].x),2)+pow((VEGE_Y-VEGELoc[VEGEcounter2].y),2) < 2*VegeRadius*VegeRadius)
                                        #ifdef(Verbose)
                                                #warning "failed\n"
                                        #end
                                        #local VEGE_OK = false;
                                        #local VEGEcounter2 = VEGEcounter; //leave loop
                                        #local VEGEReject = VEGEReject + 1;
                                #end
                                #local VEGEcounter2 = VEGEcounter2 + 1;
                        #end
                
                #if(VEGE_OK)
                        #local VEGELoc[VEGEcounter] = VEGE_Temp;
                        #ifdef(Verbose)
                                #warning "Passed\n"
                        #end
                #end
                #end

        #end //End do

        #if(VegeTypesGauss)
                #local VEGENUM[VEGEcounter] = int(RandGauss()*VEGETypes); //Select type of object
        #else
                #local VEGENUM[VEGEcounter] = int(rand(VegeSeed)*VEGETypes); //Select type of object
        #end
        #local VEGE_SCALE = rand(VegeSeed)*VEGE_SCALE_RANGE+VEGE_SCALE_MIN;
        #if(VegeRotate)
                #local VEGERotate = rand(VegeSeed)*360;
        #else
                #local VEGERotate = 0;
        #end
        
    #local VEGETrans[VEGEcounter] = transform{
        scale VEGE_SCALE
        rotate VEGERotate*SkyVector
        #if(VegeAlign)
                Reorient(SkyVector, VEGE_Vect)
        #end
        translate (VEGE_X*X_DIR)+(VEGE_Y*Y_DIR)+(abs(VEGE_Z)*SkyVector)
    }
    
        #local VEGEcounter = VEGEcounter + 1;
        #ifdef(Verbose)
                #warning concat( "Object number ", str(VEGEcounter, 0, 0), " placed\n" )
        #end
#end
#ifdef(Verbose)
        #warning "\nStarting output \n"
#end
//Output the objects
#declare VEGEcounter = 0;
#while(VEGEcounter < VegeNumber )
        object{VegeList[VEGENUM[VEGEcounter]] transform VEGETrans[VEGEcounter] }
        #ifdef(Verbose)
                #warning concat( "Object number ", str(VEGEcounter, 0, 0), " of type ", str(VEGENUM[VEGEcounter],0,0), " placed\n" )
        #end
        #declare VEGEcounter = VEGEcounter + 1;
#end
#warning concat("\nObject Generation Complete - ", str(VegeNumber,0,0), " objects\n")
#ifdef(Verbose)
        #warning concat( "Rejected ", str(VEGEReject,0,0), " object positions for collisions\n")
        #warning concat( "Rejected ", str(VEGEHeightR,0,0), " object positions for height problems\n")
        #warning concat( "Rejected ", str(VEGEPosR,0,0), " object positions in exclusion zones\n")
        #warning concat( "Rejected ", str(VEGEOtherR,0,0), " object positions for other reasons\n")
#end
